/****************************************************************************
 *			Tuning and Analysis Utilities
 *			http://www.cs.uoregon.edu/research/paracomp/tau
 ****************************************************************************
 * Copyright (c) 1997-2006
 *    Department of Computer and Information Science, University of Oregon
 *    Advanced Computing Laboratory, Los Alamos National Laboratory
 *    Research Center Juelich, ZAM Germany	
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Wyatt Spear - initial API and implementation
 ****************************************************************************/
package org.eclipse.ptp.tau.core.internal;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Map;
// import java.util.Vector;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.ui.AbstractLaunchConfigurationTab;
import org.eclipse.debug.ui.ILaunchConfigurationDialog;
import org.eclipse.debug.ui.ILaunchConfigurationTab;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.window.Window;
// import org.eclipse.ptp.internal.core.CoreMessages;
// import org.eclipse.ptp.launch.ui.PLaunchConfigurationTab;
import org.eclipse.ptp.tau.core.internal.LaunchImages;
import org.eclipse.ptp.tau.options.TAUOptionsPlugin;
import org.eclipse.ptp.tau.papiselect.PapiListSelectionDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

/**
 * 
 */
public class AnalysisTab extends AbstractLaunchConfigurationTab {

	protected class CheckItem {

		CheckItem() {
		}

		CheckItem(String cmd, String bText, String tText, String cString,
				boolean def) {
			//Default State
			defState = def;
			//Literal appearance in makefile
			makeCmd = cmd;
			//Title on button
			buttonText = bText;
			//Tool tip
			toolText = tText;
			//Configuration constant title
			confString = cString;
		}

		/*
		 * checkUnit(String cmd, boolean def){ defState=def; makeCmd=cmd; }
		 */
		protected Button unitCheck;

		protected String makeCmd;

		protected String buttonText;

		protected String toolText;

		protected String confString;

		protected boolean defState;
	}

	protected void TauOptBrowse(CheckOption opt){
		if(opt.type==CheckOption.DIR){
			DirectoryDialog dialog = new DirectoryDialog(getShell());
			dialog.setText(opt.toolText);

			String correctPath = opt.argbox.getText();// getFieldContent(tauArch.getText());
			if (correctPath != null) {
				File path = new File(correctPath);
				if (path.exists()) {
					dialog.setFilterPath(path.isFile() ? correctPath : path
							.getParent());
				}
			}

			String selectedPath = dialog.open();
			if (selectedPath != null) {
				opt.argbox.setText(selectedPath);
			}
		}
		else
		if(opt.type==CheckOption.FILE)
		{
			FileDialog dialog = new FileDialog(getShell());
			dialog.setText(opt.toolText);

			String correctPath = opt.argbox.getText();// getFieldContent(tauArch.getText());
			if (correctPath != null) {
				File path = new File(correctPath);
				if (path.exists()) {
					dialog.setFilterPath(path.isFile() ? correctPath : path
							.getParent());
				}
			}

			String selectedPath = dialog.open();
			if (selectedPath != null) {
				opt.argbox.setText(selectedPath);
			}
		}
		
	}
	
	protected void TauOptUpdate(){
		//if(tauCompilerOptions==null)
		tauCompilerOptions="-tau_options=\'";
		for(int i=0;i<tauopts.length;i++){
			if(tauopts[i].unitCheck.getSelection())
				tauCompilerOptions+=tauopts[i].optionLine;
		}
		tauCompilerOptions+='\'';
		//System.out.println(tauCompilerOptions);
	}
	
	protected void TauOptArgUpdate(CheckOption opt){
		if(opt.type>0)
		{
			opt.optionLine=opt.makeCmd+"=\""+opt.argbox.getText()+"\" ";
			//if(opt.unitCheck.getSelection())
				TauOptUpdate();
		}
	}
	
	protected class MakeBrowseListener extends SelectionAdapter{
public void widgetSelected(SelectionEvent e) {

	Object source = e.getSource();
	for (int i = 0; i < tauopts.length; i++) {
		if (source == tauopts[i].browser) {
			
				TauOptBrowse(tauopts[i]);
				break;
			
		}
	}
}

}
	
	protected class MakeCheckListener extends SelectionAdapter implements
			ModifyListener, IPropertyChangeListener {
		public void widgetSelected(SelectionEvent e) {

			//Object source = e.getSource();
			TauOptUpdate();
			if(showComOpts!=null&&tauCompilerOptions!=null)
			{
				showComOpts.setText(tauCompilerOptions);
				//showComOpts.pack();
			}
			updateLaunchConfigurationDialog();
			/*for (int i = 0; i < tauopts.length; i++) {
				if (source == tauopts[i].unitCheck) {
					TauOptUpdate();
					break;
				}
			}*/
		}

		public void propertyChange(PropertyChangeEvent event) {
			//updateLaunchConfigurationDialog();
			TauOptUpdate();
		}

		public void modifyText(ModifyEvent evt) {
			Object source = evt.getSource();
			
			for (int i = 0; i < tauopts.length; i++) {
				if (source.equals(tauopts[i].argbox)) {
					TauOptArgUpdate(tauopts[i]);
					if(showComOpts!=null&&tauCompilerOptions!=null)
					{
						showComOpts.setText(tauCompilerOptions);
						//showComOpts.pack();
					}
					updateLaunchConfigurationDialog();
					break;
				}
			}
		}
	}

	protected class CheckOption extends CheckItem {

		public static final int BOOL = 0;

		public static final int TEXT = 1;

		public static final int DIR = 2;

		public static final int FILE = 3;

		protected String optionLine;

		protected Text argbox;

		protected Button browser;
		protected String defArg;
		protected String argConfString;

		//protected Composite argComp;
		//0 = bool, 1=arg, 2=directory, 4=file
		protected int type;

		private void setup(String name, String description, int kind) {
			toolText = description;
			defState = false;
			defArg="";
			
			argConfString=name.toUpperCase()+"_ARGUMENT_SAVED";
			confString=name.toUpperCase()+"_BUTTON_STATE";
			
			//System.out.println(argConfString + " " + confString);
			
			buttonText = name;
			//while(buttonText.length()<20){
			//	buttonText+=" ";
			//}
			makeCmd = " -opt" + name;
			type = kind;
			
			if (type > 3)
				type = 0;
			
			optionLine=makeCmd;
			if(type>0)
				optionLine+="=\"\"";
			optionLine+=' ';
		}

		CheckOption(String name, String description, int kind) {
			setup(name, description, kind);
		}

		CheckOption(String name, String description) {
			setup(name, description, 0);
		}

		public void makeItem(Composite comp) {
			if (type == BOOL) {
				unitCheck = createCheckButton(comp, buttonText);
				unitCheck.setToolTipText(toolText);
				new Label(comp, SWT.NULL);
				new Label(comp, SWT.NULL);
			} else if (type == TEXT) {
				//argComp = new Composite(comp, SWT.NONE);
				//argComp.setLayout(createGridLayout(2, false, 0, 0));
				//argComp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				unitCheck = createCheckButton(comp, buttonText);
				unitCheck.setToolTipText(toolText);

				argbox = new Text(comp, SWT.BORDER | SWT.SINGLE);
				argbox.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				argbox.addModifyListener(checkListener);

				new Label(comp, SWT.NULL);
			} else {
				//argComp = new Composite(comp, SWT.NONE);
				//argComp.setLayout(createGridLayout(3, false, 0, 0));
				//argComp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				unitCheck = createCheckButton(comp, buttonText);
				unitCheck.setToolTipText(toolText);

				argbox = new Text(comp, SWT.BORDER | SWT.SINGLE);
				argbox.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

				argbox.addModifyListener(checkListener);
				browser = createPushButton(comp, "Browse", null);
				browser.addSelectionListener(browseListener);
				//if (type == DIR) {
					//					browser.addSelectionListener(listener);
				//} else if (type == FILE) {
					//browser.addSelectionListener(listener);
				//}
			}
			unitCheck.addSelectionListener(checkListener);
		}
	}

	// Vector checkB=new Vector();//new checkUnit[5];
	// ; //.add(new checkUnit("mpi","MPI","",true));
	// checkButtons[0] =
	protected CheckItem checks[] = {
			new CheckItem("mpi", "MPI", "",
					ITAULaunchConfigurationConstants.MPI, true),
			new CheckItem("callpath", "Callpath Profiling", "",
					ITAULaunchConfigurationConstants.CALLPATH, false),
			new CheckItem("phase", "Phase Based Profiling", "",
					ITAULaunchConfigurationConstants.PHASE, false),
			new CheckItem("memory", "Memory Profiling", "",
					ITAULaunchConfigurationConstants.MEMORY, false),
			new CheckItem("opari", "OPARI", "",
					ITAULaunchConfigurationConstants.OPARI, false),
			new CheckItem("openmp", "OpenMP", "",
					ITAULaunchConfigurationConstants.OPENMP, false),
			new CheckItem("epilog", "Epilog", "",
					ITAULaunchConfigurationConstants.EPILOG, false),
			new CheckItem("vampir", "VampirTrace", "",
					ITAULaunchConfigurationConstants.VAMPIRTRACE, false),
			new CheckItem("papi", "PAPI", "",
					ITAULaunchConfigurationConstants.PAPI, false),
			// Papi is entry 8 (needed for papi composite/MULTI)
			new CheckItem("perf", "Perflib", "",
					ITAULaunchConfigurationConstants.PERF, false),
			new CheckItem("trace", "Trace", "",
					ITAULaunchConfigurationConstants.TRACE, false), };

	protected int papiIndex = 8;

	protected  CheckOption tauopts[]={
			new CheckOption("Verbose",                   "Turn on verbose debugging message"),
			new CheckOption("KeepFiles",                 "Does not remove intermediate .pdb and .inst.* files"),
			new CheckOption("TauSelectFile",          "Specify selective instrumentation file for tau_instrumentor",CheckOption.FILE),
			new CheckOption("DetectMemoryLeaks",         "Track mallocs/frees using TAU's memory wrapper"),
			new CheckOption("PdtGnuFortranParser",       "Specify the GNU gfortran PDT parser gfparse instead of f95parse"),
			new CheckOption("PdtDir",                 "PDT architecture directory. Typically $(PDTDIR)/$(PDTARCHDIR)",CheckOption.DIR),
			new CheckOption("PdtF95Opts",             "Options for Fortran parser in PDT (f95parse)",CheckOption.TEXT),
			new CheckOption("PdtF95Reset",            "Reset options to the Fortran parser to the given list",CheckOption.TEXT),
			new CheckOption("PdtCOpts",               "Options for C parser in PDT (cparse). Typically $(TAU_MPI_INCLUDE) $(TAU_INCLUDE) $(TAU_DEFS)",CheckOption.TEXT),
			new CheckOption("PdtCReset",              "Reset options to the C parser to the given list",CheckOption.TEXT),
			new CheckOption("PdtCxxOpts",             "Options for C++ parser in PDT (cxxparse). Typically $(TAU_MPI_INCLUDE) $(TAU_INCLUDE) $(TAU_DEFS)",CheckOption.TEXT),
			new CheckOption("PdtCReset",              "Reset options to the C++ parser to the given list",CheckOption.TEXT),
			new CheckOption("PdtF90Parser",           "Specify a different Fortran parser. For e.g., f90parse instead of f95parse",CheckOption.TEXT),
			new CheckOption("PdtUser",                "Optional arguments for parsing source code",CheckOption.TEXT),
			new CheckOption("TauInstr",               "Specify location of tau_instrumentor. Typically $(TAUROOT)/$(CONFIG_ARCH)/bin/tau_instrumentor",CheckOption.DIR),//File?
			new CheckOption("PreProcess",                "Preprocess the source code before parsing. Uses /usr/bin/cpp -P by default."),
			new CheckOption("CPP",                    "Specify an alternative preprocessor and pre-process the sources.",CheckOption.TEXT),
			new CheckOption("CPPOpts",                "Specify additional options to the C pre-processor.",CheckOption.TEXT),
			new CheckOption("CPPReset",               "Reset C preprocessor options to the specified list.",CheckOption.TEXT),
			new CheckOption("PDBFile",                "Specify PDB file for tau_instrumentor. Skips parsing stage.",CheckOption.FILE),
			new CheckOption("Tau",                    "Specify options for tau_instrumentor",CheckOption.TEXT),
			new CheckOption("Compile",                "Options passed to the compiler by the user.",CheckOption.TEXT),
			new CheckOption("TauDefs",                "Options passed to the compiler by TAU. Typically $(TAU_DEFS)",CheckOption.TEXT),
			new CheckOption("TauIncludes",            "Options passed to the compiler by TAU. Typically $(TAU_MPI_INCLUDE) $(TAU_INCLUDE)",CheckOption.TEXT),
			new CheckOption("IncludeMemory",          "Flags for replacement of malloc/free. Typically -I$(TAU_DIR)/include/Memory",CheckOption.TEXT),
			new CheckOption("Reset",                  "Reset options to the compiler to the given list",CheckOption.TEXT),
			new CheckOption("Linking",                "Options passed to the linker. Typically $(TAU_MPI_FLIBS) $(TAU_LIBS) $(TAU_CXXLIBS)",CheckOption.TEXT),
			new CheckOption("LinkReset",              "Reset options to the linker to the given list",CheckOption.TEXT),
			new CheckOption("TauCC",                  "Specifies the C compiler used by TAU",CheckOption.TEXT),
			new CheckOption("OpariTool",              "Specifies the location of the Opari tool",CheckOption.DIR),//file?
			new CheckOption("OpariDir",               "Specifies the location of the Opari directory",CheckOption.DIR),
			new CheckOption("OpariOpts",              "Specifies optional arguments to the Opari tool",CheckOption.TEXT),
			new CheckOption("OpariReset",             "Resets options passed to the Opari tool",CheckOption.TEXT),
			new CheckOption("NoMpi",                     "Removes -l*mpi* libraries during linking (default)"),
			new CheckOption("Mpi",                       "Does not remove -l*mpi* libraries during linking"),
			new CheckOption("NoRevert",                  "Exit on error. Does not revert to the original compilation rule on error."),
			new CheckOption("Revert",                    "Revert to the original compilation rule on error (default)."),
			new CheckOption("AppCC",                  "Specifies the fallback C compiler.",CheckOption.TEXT),
			new CheckOption("AppCXX",                 "Specifies the fallback C++ compiler.",CheckOption.TEXT),
			new CheckOption("AppF90",                 "Specifies the fallback F90 compiler.",CheckOption.TEXT)
	};

	// protected Button papiCheck;
	protected Button papiSelect;

	protected Button papiCountRadios[];

	protected Composite papiComp;

	// protected Button perfCheck;
	// protected Button traceCheck;
	// protected Button icpcCheck;
	protected Composite selComp;

	protected Label selectLabel;

	protected Button selectRadios[];

	protected Text compiler;

	protected Button buildonlyCheck;

	protected Button nocleanCheck;

	protected Button keepprofsCheck;

	protected Button portalCheck;

	protected Text tauArch = null;

	protected Button browseSelfileButton = null;

	protected LinkedHashSet allmakefiles = null;

	protected LinkedHashSet selmakefiles = null;

	protected LinkedHashSet allopts = null;

	protected LinkedHashSet selopts = null;

	protected Combo makecombo = null;

	protected String selmakefile = null;
	
	protected String tauCompilerOptions = null;

	private String tlpath = null;

	protected Map archvarmap = null;// PAPI Map entries

	protected Map varmap = null;
	
	protected Text showComOpts=null;

	protected MakeCheckListener checkListener =new MakeCheckListener();
	protected MakeBrowseListener browseListener = new MakeBrowseListener();
	
	// protected Map environ = null;//Current environment
	// protected int numvars=0;

	protected class WidgetListener extends SelectionAdapter implements
			ModifyListener, IPropertyChangeListener {
		public void widgetSelected(SelectionEvent e) {

			Object source = e.getSource();

			if (source == makecombo) {
				selmakefile = makecombo.getItem(makecombo.getSelectionIndex());
				if ((selmakefile.indexOf("-papi") > 0)
						&& (selmakefile.indexOf("-multiplecounters") > 0)) {
					papiSelect.setEnabled(true);
				} else {
					papiSelect.setEnabled(false);
				}
			}

			if (source == browseSelfileButton) {
				handleSelfileBrowseButtonSelected();
			} else if (source == papiSelect) {
				handlePapiSelect();
			} else if (source.equals(selectRadios[2])) {
				if (!selectRadios[2].getSelection()) {
					selComp.setEnabled(false);
					tauArch.setEnabled(false);
					tauArch.setEnabled(false);
				} else {
					selComp.setEnabled(true);
					tauArch.setEnabled(true);
					tauArch.setEnabled(true);
				}
			} else {
				for (int i = 0; i < checks.length; i++) {
					if (source == checks[i].unitCheck) {
						if (((Button) source).getSelection()) {
							selopts.add(checks[i].makeCmd);
						} else {
							selopts.remove(checks[i].makeCmd);
						}
						initMakeCombo();
						reinitMakeChecks();
					}
				}
			}
			updateLaunchConfigurationDialog();
		}

		public void propertyChange(PropertyChangeEvent event) {
			updateLaunchConfigurationDialog();
		}

		public void modifyText(ModifyEvent evt) {
			Object source = evt.getSource();
			if (source == tauArch) {
				// setupMakefiles();
				// tcombo.select(0);
			}
			// System.out.println("ChangeText");
			updateLaunchConfigurationDialog();
		}
	}

	protected WidgetListener listener = new WidgetListener();

	private void reinitMakeChecks() {

		LinkedHashSet goodopts = new LinkedHashSet(allopts.size());
		String holdmake = null;
		String check = null;

		if ((selopts == null) || (selopts.size() == 0)) {
			goodopts.addAll(allopts);
		} else {

			Iterator makes = allmakefiles.iterator();
			boolean allgood = true;
			while (makes.hasNext()) {
				holdmake = (String) makes.next();

				Iterator opts = selopts.iterator();

				while (opts.hasNext()) {
					check = (String) opts.next();

					if (holdmake.indexOf("-" + check) <= 0) {
						allgood = false;
						break;
					}
				}
				if (allgood == true) {
					goodopts.addAll(Arrays.asList(holdmake.split("-")));
				} else {
					allgood = true;
				}
			}
		}

		for (int i = 0; i < checks.length; i++) {
			if (!goodopts.contains(checks[i].makeCmd)) {
				checks[i].unitCheck.setEnabled(false);
			} else {
				checks[i].unitCheck.setEnabled(true);
			}
		}
	}

	private void initMakeChecks() {
		if ((allmakefiles == null) || (allmakefiles.size() == 0)
				|| (allopts == null) || (allopts.size() == 0)) {
			for (int i = 0; i < checks.length; i++) {
				checks[i].unitCheck.setEnabled(false);
			}
			// papiSelect.setEnabled(false);
			return;
		}

		for (int i = 0; i < checks.length; i++) {
			if (!allopts.contains(checks[i].makeCmd)) {
				checks[i].unitCheck.setEnabled(false);
			}
		}
	}

	private void initMakefiles() {
		IPreferenceStore pstore = TAUOptionsPlugin.getDefault()
				.getPreferenceStore();
		String archpath = pstore.getString("TAUCDTArchPath");

		File[] mfiles = testTAUEnv(archpath);

		if ((mfiles == null) || (mfiles.length == 0)) {
			String checkArch = checkTAUEnvPath();
			if (checkArch != null) {
				mfiles = testTAUEnv(checkArch);
			} else
				checkArch = archpath;

			while ((mfiles == null || mfiles.length == 0) && checkArch != null) {
				checkArch = askTAUPath(checkArch);
				mfiles = testTAUEnv(checkArch);
			}

			if (checkArch == null) {
				checkArch = "/";
				mfiles = testTAUEnv(checkArch);
			}

			pstore.setValue("TAUCDTArchPath", checkArch);
		}

		allmakefiles = new LinkedHashSet();
		allopts = new LinkedHashSet();
		String name = null;
		for (int i = 0; i < mfiles.length; i++) {
			name = mfiles[i].getName();
			allmakefiles.add(name);
			allopts.addAll(Arrays.asList(name.split("-")));
		}
		allopts.remove("Makefile.tau");
	}

	/*
	 * Given a directory (presumably a tau arch directory) this looks in the lib
	 * subdirectory and returns a list of all Makefile.tau... files with -pdt
	 * */
	private File[] testTAUEnv(String archpath) {
		class makefilter implements FilenameFilter {
			public boolean accept(File dir, String name) {
				if ((name.indexOf("Makefile.tau") != 0)
						|| (name.indexOf("-pdt") <= 0)) {
					return false;
				}
				/*Only include papi makefiles built with multiplecounters*/
				if (name.indexOf("-multiplecounters") <= 0
						&& (name.indexOf("-papi") > 0)) {
					return false;
				}

				return true;
			}
		}
		tlpath = archpath + File.separator + "lib";
		File taulib = new File(tlpath);
		File[] mfiles = null;
		makefilter mfilter = new makefilter();
		if (taulib.exists()) {
			mfiles = taulib.listFiles(mfilter);
		}

		return mfiles;
	}

	/*This returns the name of the parent of the directory containing pprof
	 * which should be the tau arch directory, if one is present*/
	private String checkTAUEnvPath() {
		String pPath = null;
		try {
			Process p = Runtime.getRuntime().exec("which tau_merge");
			BufferedReader reader = new BufferedReader(new InputStreamReader(p
					.getInputStream()));
			pPath = reader.readLine();
			while (reader.readLine() != null) {
			}
			reader.close();
			reader = new BufferedReader(new InputStreamReader(p
					.getErrorStream()));
			while (reader.readLine() != null) {
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		if (pPath == null)
			return null;
		File test = new File(pPath);
		if (test.exists()) {
			return test.getParentFile().getParent();
		} else
			return null;
	}

	/*Given a string as a starting point, this asks the user for
	 * the location of a TAU arch directory*/
	private String askTAUPath(String archpath) {
		DirectoryDialog dialog = new DirectoryDialog(PlatformUI.getWorkbench()
				.getDisplay().getActiveShell());
		dialog.setText("Select TAU Arch Directory");
		dialog.setMessage("You must select a valid TAU architecture "
				+ "directory.  Such a directory should be created "
				+ "when you configure and install TAU.  It must "
				+ "contain least one valid stub makefile configured "
				+ "with the Program Database Toolkit (pdt)");
		if (archpath != null) {
			File path = new File(archpath);
			if (path.exists()) {
				dialog.setFilterPath(path.isFile() ? archpath : path
						.getParent());
			}
		}
		return dialog.open();
	}

	/*
	 * (re)populates the makefile combo box with those makefiles that match the
	 * selected options
	 */
	private void initMakeCombo() {
		try {
			// String oldsel="";
			// if(makecombo.getSelectionIndex()>=0)
			// oldsel=makecombo.getItem(makecombo.getSelectionIndex());
			String adding = "";

			makecombo.removeAll();
			// Put the list of valid makefiles in selmakefiles
			selectMakefiles();
			String select = "";

			// If there are valid makefiles, put each one in the fresh combobox
			if ((selmakefiles != null) && (selmakefiles.size() > 0)) {
				Iterator i = selmakefiles.iterator();
				while (i.hasNext()) {
					adding = (String) i.next();
					// We want to select the minimal (shortest) makefile by
					// default
					if ((select.length() > adding.length())
							|| select.equals("")) {
						select = adding;
					}
					makecombo.add(adding);
				}
				// If the currently selected makefile is still present, keep it.
				// Otherwise use the shortest
				if ((selmakefile != null)
						&& (makecombo.indexOf(selmakefile) >= 0)) {
					makecombo.select(makecombo.indexOf(selmakefile));
				} else {
					makecombo.select(makecombo.indexOf(select));
				}
			}
			// If there are no valid makefiles, make it known
			else {
				makecombo.add("No Valid Makefiles!");
				makecombo.select(0);
			}
			String checkforpapi = makecombo.getItem(makecombo
					.getSelectionIndex());
			// If the new makefile has the right options, activate the papi
			// selector
			if ((checkforpapi.indexOf("-papi") > 0)
					&& (checkforpapi.indexOf("-multiplecounters") > 0)) {
				papiSelect.setEnabled(true);
			} else {
				papiSelect.setEnabled(false);
			}
			makecombo.pack();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/*
	 * Populates the set selmakefiles with those makefiles in allmakefiles that
	 * contain every option in selopts
	 */
	private void selectMakefiles() {
		selmakefiles = new LinkedHashSet();
		Iterator allit = allmakefiles.iterator();
		String curmake = "";
		String curopt = "";
		// Look at each makefile individually
		while (allit.hasNext()) {
			Iterator nameit = selopts.iterator();
			// Assume the makefile meets the desired criteria
			boolean hasall = true;
			curmake = (String) allit.next();
			// Look at each option in the selected options
			while (nameit.hasNext()) {
				curopt = (String) nameit.next();
				// If the makefile is missing a required option, mark it a
				// failure and keep checking
				if (curmake.indexOf("-" + curopt) <= 0) {
					hasall = false;
					break;
				}
			}
			// If the makefile wasn't rejected, add it to the list of selectable
			// makefiles
			if (hasall) {
				selmakefiles.add(curmake);
			} else {
				hasall = true;
			}
		}
	}

	/**
	 * @see ILaunchConfigurationTab#createControl(Composite)
	 */
	public void createControl(Composite parent) {

		ScrolledComposite comp = new ScrolledComposite(parent, SWT.V_SCROLL);

		setControl(comp);

		GridLayout topLayout = new GridLayout();
		comp.setLayout(topLayout);

		TabFolder tabParent = new TabFolder(comp, SWT.BORDER);

		

		/*
		 * 
		 * Analysis Options:  TAU Makefile options and PAPI counter selection
		 * 
		 * */
		TabItem anaTab = new TabItem(tabParent, SWT.NULL);
		anaTab.setText("Analysis Options");

		Composite anaComp = new Composite(tabParent, SWT.NONE);
		anaTab.setControl(anaComp);

		anaComp.setLayout(createGridLayout(1, false, 0, 0));
		anaComp.setLayoutData(spanGridData(GridData.FILL_HORIZONTAL, 5));

		/*
		 * The actual controls of AnaComp
		 * */
		createVerticalSpacer(anaComp, 2);

		for (int i = 0; i < checks.length; i++) {
			/*Papi is a special case*/
			if (i != papiIndex) {
				checks[i].unitCheck = createCheckButton(anaComp,
						checks[i].buttonText);
				checks[i].unitCheck.setToolTipText(checks[i].toolText);
				checks[i].unitCheck.addSelectionListener(listener);
			} else {
				papiComp = new Composite(anaComp, SWT.NONE);
				papiComp.setLayout(createGridLayout(4, false, 0, 0));
				papiComp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				checks[i].unitCheck = createCheckButton(papiComp,
						checks[i].buttonText);
				checks[i].unitCheck.setToolTipText(checks[i].toolText);
				checks[i].unitCheck.addSelectionListener(listener);
				papiSelect = createPushButton(papiComp, "Select PAPI Counters",
						null);
				papiSelect
						.setToolTipText("Set PAPI COUNTER environment variables");
				papiSelect.addSelectionListener(listener);
				papiCountRadios = new Button[2];
				papiCountRadios[0] = createRadioButton(papiComp,
						"Preset Counters");
				papiCountRadios[1] = createRadioButton(papiComp,
						"Native Counters");
			}
		}
		// createVerticalSpacer(parallelComp, 2);
		/*
		 * Composite comComp=new Composite(parallelComp,SWT.NONE);
		 * comComp.setLayout(createGridLayout(2, false, 0, 0));
		 * comComp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL)); Label
		 * compLab = new Label(comComp, 0); compLab.setText("Using Compiler:");
		 * compiler = new Text(comComp, SWT.BORDER | SWT.SINGLE);
		 * compiler.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		 * compiler.addModifyListener(listener);
		 */

		Composite makeComp = new Composite(anaComp, SWT.NONE);
		makeComp.setLayout(createGridLayout(2, false, 0, 0));
		makeComp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		Label makeLab = new Label(makeComp, 0);
		makeLab.setText("Select Makefile:");
		makecombo = new Combo(makeComp, SWT.DROP_DOWN | SWT.READ_ONLY
				| SWT.BORDER);
		makecombo.addSelectionListener(listener);

		/*
		 * 
		 * TAU Compiler:  TAU Compiler options
		 * 
		 * */
		TabItem optTab = new TabItem(tabParent, SWT.NULL);
		optTab.setText("TAU Compiler");

		ScrolledComposite scrollOpt = new ScrolledComposite(tabParent,
				SWT.V_SCROLL);

		Composite optComp = new Composite(scrollOpt, SWT.NONE);
		optTab.setControl(scrollOpt);

		optComp.setLayout(createGridLayout(3, false, 0, 0));
		optComp.setLayoutData(spanGridData(GridData.FILL_HORIZONTAL, 5));

		/*
		 * The actual controls of optComp
		 * */
		
		//CheckOption thisCO;
		createVerticalSpacer(optComp, 3);
		
		showComOpts = new Text(optComp,SWT.BORDER|SWT.WRAP|SWT.MULTI|SWT.V_SCROLL);
		GridData showComGD = new GridData();
		showComGD.horizontalAlignment=GridData.FILL;
		showComGD.verticalAlignment=GridData.FILL;
		//showComGD.verticalSpan=2;
		showComGD.horizontalSpan=3;
		showComGD.grabExcessHorizontalSpace=true;
		showComGD.grabExcessVerticalSpace=true;
		//showComOpts.getLineHeight()*4;
		//System.out.println(showComOpts.getLineHeight()*4);
		showComGD.minimumHeight=showComOpts.getLineHeight()*3;
		
		//showComGD.
		//showComOpts.set .setSize(SWT.DEFAULT, 200);
		showComOpts.setLayoutData(showComGD);//new GridData(GridData.FILL_HORIZONTAL)
		//showComOpts.set
		//showComOpts.set
		showComOpts.setEditable(false);
		//showComOpts.setText("Hey hey XXXXXXXXXXX refreshRuntimeSystems(false), isInitialized():true XXXXXXXXXXX refreshRuntimeSystems(false), isInitialized():true XXXXXXXXXXX refreshRuntimeSystems(false), isInitialized():true");//tauCompilerOptions
		//comOpts.set
		int optlen=tauopts.length;
		for (int i = 0; i < optlen; i++) {
			tauopts[i].makeItem(optComp);
		}
		
		createVerticalSpacer(optComp, 3);
		optComp.pack();
		int optCompHeight=optComp.computeSize(SWT.DEFAULT, SWT.DEFAULT).y;//optComp.getClientArea().height;
		//System.out.println(optCompHeight);
		
		scrollOpt.setContent(optComp);
		scrollOpt.setMinSize(400, optCompHeight);//1350+showComOpts.getLineHeight());
		scrollOpt.setExpandHorizontal(true);
		scrollOpt.setExpandVertical(true);

		/*
		 * 
		 * Selective Instrumentation
		 * 
		 * */
		TabItem selinstTab = new TabItem(tabParent, SWT.NULL);
		selinstTab.setText("Selective Instrumentation");

		Composite selinstComp = new Composite(tabParent, SWT.NONE);
		selinstTab.setControl(selinstComp);

		selinstComp.setLayout(createGridLayout(1, false, 0, 0));
		selinstComp.setLayoutData(spanGridData(GridData.FILL_HORIZONTAL, 5));

		/*
		 * The actual controls of selinstTab
		 * */
		//		createVerticalSpacer(parallelComp, 1);
		//new Label(parallelComp, SWT.SEPARATOR|SWT.HORIZONTAL);
		createVerticalSpacer(selinstComp, 1);

		//selectLabel = new Label(parallelComp, SWT.SHADOW_NONE);
		//selectLabel.setText("Selective Instrumentation");
		selectRadios = new Button[4];

		selectRadios[0] = createRadioButton(selinstComp, "None");
		selectRadios[0].setToolTipText("Do not use selective instrumentation.");
		selectRadios[1] = createRadioButton(selinstComp, "Internal");
		selectRadios[1]
				.setToolTipText("Use the selective instrumentation file generated"
						+ " by selective instrumentation commands in the workspace.");
		selectRadios[2] = createRadioButton(selinstComp, "User Defined");
		selectRadios[2]
				.setToolTipText("Specify a pre-existing selective instrumentation "
						+ "file.");

		selComp = new Composite(selinstComp, SWT.NONE);
		selComp.setLayout(createGridLayout(2, false, 0, 0));
		selComp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		tauArch = new Text(selComp, SWT.BORDER | SWT.SINGLE);
		tauArch.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		tauArch.addModifyListener(listener);
		browseSelfileButton = createPushButton(selComp, "Browse", null);
		browseSelfileButton.addSelectionListener(listener);

		selectRadios[3] = createRadioButton(selinstComp, "Automatic");
		selectRadios[3].setEnabled(false);
		for (int i = 0; i < selectRadios.length; i++) {
			selectRadios[i].addSelectionListener(listener);
		}

		/*
		 * 
		 * Data Collection: Storage and management of output data
		 * 
		 * */
		TabItem dataTab = new TabItem(tabParent, SWT.NULL);
		dataTab.setText("Data Collection");

		Composite dataComp = new Composite(tabParent, SWT.NONE);
		dataTab.setControl(dataComp);

		dataComp.setLayout(createGridLayout(1, false, 0, 0));
		dataComp.setLayoutData(spanGridData(GridData.FILL_HORIZONTAL, 5));
		/*
		 * The actual controls of dataTab
		 * */
		//createVerticalSpacer(parallelComp, 1);
		//new Label(parallelComp, SWT.SEPARATOR|SWT.HORIZONTAL);
		createVerticalSpacer(dataComp, 1);

		buildonlyCheck = createCheckButton(dataComp,
				"Build the instrumented executable but do not launch it");
		buildonlyCheck.addSelectionListener(listener);
		nocleanCheck = createCheckButton(dataComp,
				"Keep instrumented executable");
		nocleanCheck.addSelectionListener(listener);
		keepprofsCheck = createCheckButton(dataComp, "Keep profiles");
		keepprofsCheck.addSelectionListener(listener);

		portalCheck = createCheckButton(dataComp,
				"Upload profile data to TAU Portal");
		portalCheck.addSelectionListener(listener);
		
		
		
		comp.setContent(tabParent);
		//int mainCompHeight=tabParent.computeSize(SWT.DEFAULT, SWT.DEFAULT).y;
		//System.out.println(mainCompHeight);
		comp.setMinSize(400, 450);
		comp.setExpandHorizontal(true);
		comp.setExpandVertical(true);
	}

	public void updateComboFromSelection() {
		System.out.println("change startup");
	}

	/**
	 * Defaults are empty.
	 * 
	 * @see ILaunchConfigurationTab#setDefaults(ILaunchConfigurationWorkingCopy)
	 */
	public void setDefaults(ILaunchConfigurationWorkingCopy configuration) {
		configuration.setAttribute(ITAULaunchConfigurationConstants.MPI,
				ITAULaunchConfigurationConstants.MPI_DEF);
		configuration.setAttribute(ITAULaunchConfigurationConstants.CALLPATH,
				ITAULaunchConfigurationConstants.CALLPATH_DEF);
		configuration.setAttribute(ITAULaunchConfigurationConstants.MEMORY,
				ITAULaunchConfigurationConstants.MEMORY_DEF);
		configuration.setAttribute(ITAULaunchConfigurationConstants.PAPI,
				ITAULaunchConfigurationConstants.PAPI_DEF);
		configuration.setAttribute(ITAULaunchConfigurationConstants.PERF,
				ITAULaunchConfigurationConstants.PERF_DEF);
		configuration.setAttribute(ITAULaunchConfigurationConstants.TRACE,
				ITAULaunchConfigurationConstants.TRACE_DEF);
		configuration.setAttribute(ITAULaunchConfigurationConstants.PHASE,
				ITAULaunchConfigurationConstants.PHASE_DEF);
		configuration.setAttribute(ITAULaunchConfigurationConstants.COMPILER,
				ITAULaunchConfigurationConstants.COMPILER_DEF);

		configuration.setAttribute(ITAULaunchConfigurationConstants.EPILOG,
				ITAULaunchConfigurationConstants.EPILOG_DEF);
		configuration.setAttribute(
				ITAULaunchConfigurationConstants.VAMPIRTRACE,
				ITAULaunchConfigurationConstants.VAMPIRTRACE_DEF);

		configuration.setAttribute(ITAULaunchConfigurationConstants.NOCLEAN,
				ITAULaunchConfigurationConstants.NOCLEAN_DEF);
		configuration.setAttribute(ITAULaunchConfigurationConstants.KEEPPROFS,
				ITAULaunchConfigurationConstants.KEEPPROFS_DEF);

		configuration.setAttribute(ITAULaunchConfigurationConstants.SELECT, 0);
		configuration.setAttribute(
				ITAULaunchConfigurationConstants.SELECT_FILE, "");

		configuration.setAttribute(ITAULaunchConfigurationConstants.ENVVARS,
				(Map) null);

		configuration.setAttribute(ITAULaunchConfigurationConstants.MAKEFILE,
				"");
		
		for(int i=0;i<tauopts.length;i++){
			configuration.setAttribute(tauopts[i].confString, tauopts[i].defState);
			if(tauopts[i].type!=CheckOption.BOOL)
				configuration.setAttribute(tauopts[i].argConfString, tauopts[i].defArg);
		}

	}

	/**
	 * @see ILaunchConfigurationTab#initializeFrom(ILaunchConfiguration)
	 */
	public void initializeFrom(ILaunchConfiguration configuration) {
		try {

			selopts = new LinkedHashSet();

			initMakefiles();
			initMakeChecks();

			for (int i = 0; i < checks.length; i++) {
				checks[i].unitCheck.setSelection(configuration.getAttribute(
						checks[i].confString, checks[i].defState));
				if (checks[i].unitCheck.getSelection()
						&& checks[i].unitCheck.getEnabled()) {
					selopts.add(checks[i].makeCmd);
				}
			}

			//tauCompilerOptions = "-tau_options=''";
			TauOptUpdate();
			showComOpts.setText(tauCompilerOptions);
			
			
			for(int i=0;i<tauopts.length;i++){
				tauopts[i].unitCheck.setSelection(configuration.getAttribute(tauopts[i].confString,tauopts[i].defState));
				if(tauopts[i].type!=CheckOption.BOOL)
					tauopts[i].argbox.setText(configuration.getAttribute(tauopts[i].argConfString, tauopts[i].defArg));
			}

			// int papiSelected =
			// configuration.getAttribute(ITAULaunchConfigurationConstants.
			//PAPISELECT,0);
			papiCountRadios[configuration.getAttribute(
					ITAULaunchConfigurationConstants.PAPISELECT, 0)]
					.setSelection(true);

			/*
			 * compiler.setText(configuration.getAttribute
			 * (ITAULaunchConfigurationConstants.COMPILER,
			 * ""));
			 * 
			 * if(compiler.getText().length()>2) {
			 * selopts.add(compiler.getText().trim()); }
			 */

			selmakefile = configuration.getAttribute(
					ITAULaunchConfigurationConstants.MAKEFILE, (String) null);

			initMakeCombo();
			reinitMakeChecks();
			int selected = configuration.getAttribute(
					ITAULaunchConfigurationConstants.SELECT, 0);

			selectRadios[selected].setSelection(true);

			tauArch.setText(configuration.getAttribute(
					ITAULaunchConfigurationConstants.SELECT_FILE, ""));

			if (!selectRadios[2].getSelection()) {
				selComp.setEnabled(false);
				tauArch.setEnabled(false);
				tauArch.setEnabled(false);
			}

			buildonlyCheck.setSelection(configuration.getAttribute(
					ITAULaunchConfigurationConstants.BUILDONLY, false));
			nocleanCheck.setSelection(configuration.getAttribute(
					ITAULaunchConfigurationConstants.NOCLEAN, false));
			keepprofsCheck.setSelection(configuration.getAttribute(
					ITAULaunchConfigurationConstants.KEEPPROFS, false));

			portalCheck.setSelection(configuration.getAttribute(
					ITAULaunchConfigurationConstants.PORTAL, false));

			varmap = archvarmap = configuration.getAttribute(
					ITAULaunchConfigurationConstants.ENVVARS, (Map) null);

			// if(varmap!=null)numvars=varmap.size();

		} catch (CoreException e) {
			setErrorMessage("Core Exception while initializing Analysis tab: "
					+ e.getMessage());
		}

	}

	/**
	 * @see ILaunchConfigurationTab#performApply(ILaunchConfigurationWorkingCopy)
	 */
	public void performApply(ILaunchConfigurationWorkingCopy configuration) {
		for (int i = 0; i < checks.length; i++) {
			configuration.setAttribute(checks[i].confString,
					checks[i].unitCheck.getSelection());
		}
		
		for(int i =0; i < tauopts.length;i++){
			configuration.setAttribute(tauopts[i].confString, tauopts[i].unitCheck.getSelection());
			if(tauopts[i].type!=CheckOption.BOOL)
				configuration.setAttribute(tauopts[i].argConfString, tauopts[i].argbox.getText());
		}
		configuration.setAttribute(ITAULaunchConfigurationConstants.TAU_COMPILER_OPTIONS, tauCompilerOptions);

		// configuration.setAttribute(ITAULaunchConfigurationConstants.COMPILER,
		// compiler.getText());

		configuration.setAttribute(ITAULaunchConfigurationConstants.BUILDONLY,
				buildonlyCheck.getSelection());
		configuration.setAttribute(ITAULaunchConfigurationConstants.NOCLEAN,
				nocleanCheck.getSelection());
		configuration.setAttribute(ITAULaunchConfigurationConstants.KEEPPROFS,
				keepprofsCheck.getSelection());
		configuration.setAttribute(ITAULaunchConfigurationConstants.PORTAL,
				portalCheck.getSelection());

		if (((varmap == null) && (archvarmap != null))
				|| ((varmap != null) && (archvarmap == null))
				|| ((varmap != null) && (archvarmap != null) && !varmap
						.equals(archvarmap))) {
			Map envvars = null;

			try {
				envvars = configuration.getAttribute(
						ILaunchManager.ATTR_ENVIRONMENT_VARIABLES, (Map) null);
			} catch (CoreException e) {
				e.printStackTrace();
			}

			if ((envvars != null) && (envvars.size() > 0)
					&& (archvarmap != null) && (archvarmap.size() > 0)) {
				Iterator archit = archvarmap.keySet().iterator();
				while (archit.hasNext()) {
					envvars.remove(archit.next());
				}
			}

			if ((varmap != null) && (varmap.size() > 0)) {
				if (envvars == null) {
					envvars = new HashMap();
				}
				envvars.putAll(varmap);
			}

			configuration.setAttribute(
					ILaunchManager.ATTR_ENVIRONMENT_VARIABLES, envvars);
			// updateLaunchConfigurationDialog();
			configuration.setAttribute(
					ITAULaunchConfigurationConstants.ENVVARS, varmap);
		}

		// int papiSelected=0;
		if (papiCountRadios[0].getSelection()) {
			configuration.setAttribute(ITAULaunchConfigurationConstants.SELECT,
					0);
		} else {
			configuration.setAttribute(ITAULaunchConfigurationConstants.SELECT,
					1);
		}

		int selected = 0;
		for (int i = 0; i < selectRadios.length; i++) {
			if (selectRadios[i].getSelection()) {
				selected = i;
				break;
			}
		}
		configuration.setAttribute(ITAULaunchConfigurationConstants.SELECT,
				selected);

		configuration
				.setAttribute(ITAULaunchConfigurationConstants.SELECT_FILE,
						tauArch.getText());

		configuration.setAttribute(ITAULaunchConfigurationConstants.MAKEFILE,
				makecombo.getItem(makecombo.getSelectionIndex()));

	}

	protected String getFieldContent(IntegerFieldEditor editorField) {
		return getFieldContent(editorField.getStringValue());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#isValid
	 * (org.eclipse.debug.core.ILaunchConfiguration)
	 */
	public boolean isValid(ILaunchConfiguration configuration) {
		setErrorMessage(null);
		setMessage(null);

		// if (!numberOfProcessField.isValid()) {
		// setErrorMessage(numberOfProcessField.getErrorMessage());
		// return false;
		// }

		/*
		 * if (getFieldContent(numberOfProcessStartField) != null &&
		 * !numberOfProcessStartField.isValid()) {
		 * setErrorMessage(numberOfProcessStartField.getErrorMessage()); return
		 * false; }
		 * 
		 * if (getFieldContent(firstNodeNumberField) != null &&
		 * !firstNodeNumberField.isValid()) {
		 * setErrorMessage(firstNodeNumberField.getErrorMessage()); return
		 * false; }
		 */

		return true;
	}

	protected void handleSelfileBrowseButtonSelected() {
		FileDialog dialog = new FileDialog(getShell());
		dialog.setText("Select TAU Selective Instrumentation File");

		String correctPath = getFieldContent(tauArch.getText());
		if (correctPath != null) {
			File path = new File(correctPath);
			if (path.exists()) {
				dialog.setFilterPath(path.isFile() ? correctPath : path
						.getParent());
			}
		}

		String selectedPath = dialog.open();
		if (selectedPath != null) {
			tauArch.setText(selectedPath);
		}
	}

	private String getPapiLoc() throws FileNotFoundException {

		String papimake = tlpath + File.separator
				+ makecombo.getItem(makecombo.getSelectionIndex());

		File papimakefile = new File(papimake);
		if (!papimakefile.canRead()) {
			System.out.println("INVALID MAKEFILE FOR PAPI");
		}

		String papiline = "";
		boolean found = false;
		try {
			BufferedReader readmake = new BufferedReader(new FileReader(
					papimakefile));
			papiline = readmake.readLine();
			while (papiline != null) {
				if (papiline.indexOf("PAPIDIR=") == 0) {
					found = true;
					break;
				}
				papiline = readmake.readLine();
			}
			readmake.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		if (found) {
			papiline = papiline.substring(papiline.indexOf("=") + 1);
			File papibin = new File(papiline + File.separator + "bin"
					+ File.separator + "papi_event_chooser");
			if (!papibin.canRead()) {
				papibin = new File(papiline + File.separator + "share"
						+ File.separator + "papi" + File.separator + "utils"
						+ File.separator + "papi_event_chooser");
			}

			if (!papibin.canRead()) {
				throw new FileNotFoundException(
						"Could not locate papi utilities");
			}

			papiline = papibin.getParentFile().toString();
		} else {
			System.out.println("No PAPIDIR in Makefile!");
		}

		// System.out.println(papimake);

		// System.out.println(papiline);

		return papiline;// "/usr/local/packages/papi/bin/";
	}

	protected void handlePapiSelect() {
		try {
			LabelProvider papilab = new LabelProvider();
			ArrayContentProvider paprov = new ArrayContentProvider();

			int papiCountType = 0;
			if (papiCountRadios[1].getSelection()) {
				papiCountType = 1;
			}
			PapiListSelectionDialog papidialog = new PapiListSelectionDialog(
					getShell(), getPapiLoc(), paprov, papilab,
					"Select the PAPI counters to use with TAU", papiCountType);
			papidialog.setTitle("PAPI Counters");
			papidialog.setHelpAvailable(false);
			if ((varmap != null) && (varmap.size() > 0)) {
				// Iterator checkin = varmap.values().iterator();
				// while(checkin.hasNext()){System.out.println("Init:
				// "+checkin.next());}
				papidialog.setInitialSelections(varmap.values().toArray());
			}

			if (papidialog.open() == Window.OK) {
				Object[] selected = papidialog.getResult();
				// for(int i=0;i<selected.length;i++)
				// System.out.println(selected[i]);

				if ((selected != null) && (selected.length > 0)) {
					LinkedHashSet selset = new LinkedHashSet(Arrays
							.asList(selected));

					varmap = new HashMap(selset.size());
					varmap.put("COUNTER1", "GET_TIME_OF_DAY");
					Iterator varit = selset.iterator();
					int counter = 2;
					while (varit.hasNext()) {
						varmap.put("COUNTER" + counter, varit.next());
						counter++;
					}

					// System.out.println("Arr
					// "+varmap.entrySet().toArray().toString());
					// updateLaunchConfigurationDialog();
				} else {
					varmap = null;
				}
			}
			// this.getLaunchManager().
			// getWorkingCopy();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * @see ILaunchConfigurationTab#getName()
	 */
	public String getName() {
		return "Analysis";
		// CoreMessages.getResourceString("ParallelTab.Parallel");
	}

	/**
	 * @see ILaunchConfigurationTab#setLaunchConfigurationDialog
	 * (ILaunchConfigurationDialog)
	 */
	public void setLaunchConfigurationDialog(ILaunchConfigurationDialog dialog) {
		super.setLaunchConfigurationDialog(dialog);
	}

	/**
	 * @see ILaunchConfigurationTab#getImage()
	 */
	public Image getImage() {
		return LaunchImages.getImage("org.eclipse.ptp.tau.core.tauLogo.gif");
		// LaunchImages.IMG_ANALYSIS_TAB
	}

	protected GridLayout createGridLayout(int columns, boolean isEqual, int mh,
			int mw) {
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = columns;
		gridLayout.makeColumnsEqualWidth = isEqual;
		gridLayout.marginHeight = mh;
		gridLayout.marginWidth = mw;
		return gridLayout;
	}

	protected GridData spanGridData(int style, int space) {
		GridData gd = null;
		if (style == -1) {
			gd = new GridData();
		} else {
			gd = new GridData(style);
		}
		gd.horizontalSpan = space;
		return gd;
	}

	protected String getFieldContent(String text) {
		if ((text.trim().length() == 0) || text.equals("")) {
			return null;
		}

		return text;
	}
}