package org.eclipse.ptp.tau.launch.internal.ui;


import org.eclipse.cdt.core.model.CoreModel;
import org.eclipse.cdt.core.model.ICProject;
import org.eclipse.cdt.managedbuilder.core.IConfiguration;
import org.eclipse.cdt.managedbuilder.core.IManagedBuildInfo;
import org.eclipse.cdt.managedbuilder.core.ManagedBuildManager;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.window.Window;
import org.eclipse.ptp.core.IPTPLaunchConfigurationConstants;
import org.eclipse.ptp.launch.internal.ui.LaunchMessages;
import org.eclipse.ptp.launch.ui.PMainTab;
import org.eclipse.ptp.tau.core.internal.ITAULaunchConfigurationConstants;
import org.eclipse.ui.dialogs.ElementListSelectionDialog;

public class TAUPMainTab extends PMainTab{
	
	/**
	 * Return the ICProject corresponding to the project name in the project name text field, or
	 * null if the text does not match a project name.
	 */
	protected ICProject getCProject() {
		String projectName = projText.getText().trim();
		if (projectName.length() < 1) {
			return null;
		}
		return CoreModel.getDefault().getCModel().getCProject(projectName);
	}
	
    protected void handleApplicationButtonSelected() {
    	
    	ICProject project = getCProject();
		if (project == null) {
			MessageDialog.openInformation(getShell(), org.eclipse.cdt.launch.internal.ui.LaunchMessages.getString("CMainTab.Project_required"), //$NON-NLS-1$
					org.eclipse.cdt.launch.internal.ui.LaunchMessages.getString("CMainTab.Enter_project_before_searching_for_program")); //$NON-NLS-1$
			return;
		}
		
		IManagedBuildInfo info = ManagedBuildManager.getBuildInfo(project.getResource());
		if(info==null)
		{
			MessageDialog.openInformation(getShell(), org.eclipse.cdt.launch.internal.ui.LaunchMessages.getString("CMainTab.Project_required"), //$NON-NLS-1$
					org.eclipse.cdt.launch.internal.ui.LaunchMessages.getString("CMainTab.Enter_project_before_searching_for_program")); //$NON-NLS-1$
			return;
		}

		ILabelProvider qualifierLabelProvider = new LabelProvider() {//CElementLabelProvider()

			public String getText(Object element) {
				if (element instanceof IConfiguration) {
					IConfiguration bin = (IConfiguration)element;
					return bin.getName();
				}
				return super.getText(element);
			}
		};

		//TwoPaneElementSelector
		ElementListSelectionDialog dialog = new ElementListSelectionDialog(getShell(), qualifierLabelProvider);
		dialog.setElements(info.getManagedProject().getConfigurations());
		dialog.setMessage("Choose a build configuration"); //$NON-NLS-1$
		dialog.setTitle("Build Configuration Selection"); 
		dialog.setMultipleSelection(false);
		// dialog.set
		if (dialog.open() == Window.OK) {
			IConfiguration bconf = (IConfiguration)dialog.getFirstResult();
			appText.setText(bconf.getName());
		}
    }
	
	public void initializeFrom(ILaunchConfiguration configuration) {
        try {
            projText.setText(configuration.getAttribute(IPTPLaunchConfigurationConstants.ATTR_PROJECT_NAME, EMPTY_STRING));
            appText.setText(configuration.getAttribute(ITAULaunchConfigurationConstants.ATTR_TAUBUILD_CONFIGURATION_NAME, EMPTY_STRING));
            //stopInMainCheckButton.setSelection(configuration.getAttribute(IPTPLaunchConfigurationConstants.ATTR_STOP_IN_MAIN, false));
        } catch (CoreException e) {
            setErrorMessage(LaunchMessages.getFormattedResourceString("CommonTab.common.Exception_occurred_reading_configuration_EXCEPTION", e.getStatus().getMessage()));
        }
    }
	
    public void performApply(ILaunchConfigurationWorkingCopy configuration) {
        configuration.setAttribute(IPTPLaunchConfigurationConstants.ATTR_PROJECT_NAME, getFieldContent(projText.getText()));
        configuration.setAttribute(ITAULaunchConfigurationConstants.ATTR_TAUBUILD_CONFIGURATION_NAME, getFieldContent(appText.getText()));
        
        try {
        	
        	if(configuration.getAttribute(IPTPLaunchConfigurationConstants.ATTR_APPLICATION_NAME, EMPTY_STRING).equals(EMPTY_STRING))
        		configuration.setAttribute(IPTPLaunchConfigurationConstants.ATTR_APPLICATION_NAME,appText.getText());
//        	 TODO Add 'default' executable name from build configuration.
		} catch (CoreException e) {
			e.printStackTrace();
		}
        //configuration.setAttribute(IPTPLaunchConfigurationConstants.ATTR_STOP_IN_MAIN, stopInMainCheckButton.getSelection());
    }    
	
    
	
}
